/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package api

import "time"

// Log for contract coder
type Log interface {
	// Debug log interface
	Debug(msg string)
	Debugf(format string, args ...interface{})

	// Info log interface
	Info(msg string)
	Infof(format string, args ...interface{})

	// Warn log interface
	Warn(msg string)
	Warnf(format string, args ...interface{})

	// Error log interface
	Error(msg string)
	Errorf(format string, args ...interface{})
}

// Stub is common contract stub.
type Stub interface {
	Log

	Creator

	OracleStub

	// FuncName get the invoke function name
	FuncName() string

	// Parameters get the invoke parameters
	Parameters() [][]byte

	// TxID get the tx id for this transaction
	// Deprecated: In the next version, this interface will be deleted.
	TxID() string

	// ChainID get the chain id for this transaction
	ChainID() string

	// ContractName get the contract id for this transaction
	ContractName() string

	// TxTimestamp is to get the transaction time
	TxTimestamp() time.Time

	// EmitEvent emit contract event
	EmitEvent(name string, payload []byte) error
}

// URLEvent for url off chain data
type URLEvent struct {
	URL       string
	Timestamp int64
}

// OracleStub contain all functions of get off chain data
type OracleStub interface {
	// QueryURL query off chain data of url
	QueryURL(urlEvent *URLEvent, callback string) error
}

// SQLContract is the SQL smart contract interface .
type SQLContract interface {
	Init(stub SQLContractStub) ([]byte, error)
	Invoke(stub SQLContractStub) ([]byte, error)
}

// SQLContractStub is the SQL smart contract stub .
type SQLContractStub interface {
	Stub
	Query(query string, args ...interface{}) (Rows, error)
	Exec(query string, args ...interface{}) error
}

// Rows is the SQL rows interface .
type Rows interface {
	Next() (bool, error)
	Scan(dest ...interface{}) error
	Columns() ([]string, error)
	Close() error
}

// Contract is the leveldb interface .
type Contract interface {
	Init(stub ContractStub) ([]byte, error)
	// Invoke will abort response when QueryStream use
	Invoke(stub ContractStub) ([]byte, error)
}

// Creator contain functions of get creator info
type Creator interface {
	// GetOrg get org of creator
	GetOrg() []string

	// GetCommonName get common name of creator
	GetCommonName() string

	// GetPemCert get cert of creator
	GetPemCert() ([]byte, error)
}

// ContractStub is the leveldb stub interface .
type ContractStub interface {
	Stub

	// GetKV get value by key from state DB
	GetKV(key string) ([]byte, error)

	// PutKV put KV to DB
	// This action will only generate read write set,
	// the key value will not be put into stateDB until the transaction is validated.
	PutKV(key string, value []byte) error

	// Deprecated: In the next version, this interface will be deleted.
	// The value should be base type or implement the Marshal(v interface{}) ([]byte, error)
	// and Unmarshal(data []byte, v interface{}) error interface
	PutKVCommon(key string, value interface{}) error

	// DelKV delete key in DB
	// This action will only generate read write set, the key value will not be del until the transaction is validated.
	DelKV(key string) error

	// GetIterator get a K、V iterator for startKey and endKey.
	// Users do not need to perceive the internal buffering process,
	// even if the result are too large [startKey, endKey).
	// For example: 11--13, you will get 11, 12, not including 13;
	// 11--11,you will get nothing
	GetIterator(startKey, endKey string) (Iterator, error)

	// GetKeyHistoryIterator get history iterator of the key
	GetKeyHistoryIterator(key string) (HistoryIterator, error)

	// SaveComIndex save composite index for objectKey.
	// The mark is the common mark for the object index.
	// The attributes is the attributes for the object value
	// The objectKey is the key for the origin object
	// func GetIterator and GetKeyHistoryIterator can replace GenerateComKey、DivideComKey、GetComKeyIterator.
	SaveComIndex(indexName string, attributes []string, objectKey string) error

	// GetKVByComIndex get the object value by composite index.
	// We can get the index by mark and attributes, then we can get object key and value by index
	GetKVByComIndex(indexName string, attributes []string) (Iterator, error)

	// DelComIndexOneRow delete the index for certain object key
	DelComIndexOneRow(indexName string, attributes []string, objectKey string) error

	// SendData send stream data to wnode
	SendData(key []byte) error

	// CreateComKey generate composite key.
	CreateComKey(objectType string, attributes []string) (string, error)

	// SplitComKey is to split composite key
	SplitComKey(comKey string) (string, []string, error)

	// GetKVByPartialComKey get key value by partial composite key
	GetKVByPartialComKey(objectType string, attributes []string) (Iterator, error)

	// GetKVByPrefix get key value by Prefix
	GetKVByPrefix(prefix string) (Iterator, error)
}

// Iterator  interface for user to get key and value one by one
type Iterator interface {
	// Next is there next k/v of the iterator
	Next() bool

	// Key read key from iterator
	Key() string

	// Value read value from iterator
	Value() []byte

	// Close after read key and value from iterator, the iterator should be closed
	Close()
}

// HistoryIterator  interface for user to get key history one by one
type HistoryIterator interface {
	Iterator

	// Version returns BlockNum & TxNum
	Version() (uint64, int32)

	// TxHash returns transaction hash
	TxHash() []byte

	// IsDeleted returns whether the key has been deleted
	IsDeleted() bool

	// Timestamp returns timestamp
	Timestamp() uint64
}

// ValueSerialization when using "PutKVCommon(key string, value interface{}) error",
// the parameter "value " should be implemented with ValueSerialization interface
type ValueSerialization interface {
	Marshal() ([]byte, error)
}
